'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vueDemi = require('vue-demi');
var dom = require('@floating-ui/dom');
var vueUtility = require('@visoning/vue-utility');

// src/useAutoUpdate.ts

// src/utils/useManualEffect.ts
var createEmptyEffect = () => () => {
};
function useManualEffect(createEffect, immediate) {
  let clearEffect;
  const clear = () => {
    if (clearEffect) {
      clearEffect();
      clearEffect = void 0;
    }
  };
  const reset = (newEffect) => {
    clear();
    clearEffect = newEffect || createEffect && createEffect();
  };
  const mesure = () => {
    if (!clearEffect) {
      clearEffect = createEffect && createEffect();
    }
  };
  if (immediate) {
    mesure();
  }
  return {
    clear,
    reset,
    mesure
  };
}

// src/utils/useQualifiedRefs.ts
var defaultQualifier = (item) => item !== void 0 && item !== null;
function useQualifiedRefs(refs, handler, qualifier, options) {
  if (typeof qualifier !== "function") {
    options = options || qualifier;
    qualifier = defaultQualifier;
  }
  const detect = () => {
    const items = vueDemi.unref(refs).map(vueDemi.unref);
    if (items.some((item) => !qualifier(item))) {
      handler();
    } else {
      handler(items);
    }
  };
  if (!vueDemi.isRef(refs) && refs.every((ref2) => !vueDemi.isRef(ref2))) {
    if (options == null ? void 0 : options.immediate) {
      detect();
    }
    return {
      detect,
      mesure: vueUtility.noop,
      stop: vueUtility.noop
    };
  }
  const { clear, mesure } = useManualEffect(
    () => vueDemi.watch(() => vueDemi.unref(refs).map(vueDemi.unref), detect, options),
    true
  );
  return {
    detect,
    stop: clear,
    mesure
  };
}

// src/useAutoUpdate.ts
function useAutoUpdate(reference, floating, update, options) {
  const { reset: resetAutoUpdate, clear: clearAutoUpdate } = useManualEffect();
  const { mesure: watchElements, stop: stopWatchElements } = useQualifiedRefs([reference, floating], (qualifys) => {
    resetAutoUpdate(qualifys && dom.autoUpdate(...qualifys, update, vueDemi.unref(options)));
  });
  const handleOptionsChange = (options2) => {
    if (!options2.disabled) {
      watchElements();
    } else {
      stopWatchElements();
    }
  };
  const { mesure: watchProps, clear: stopWatchProps } = useManualEffect(
    () => vueDemi.watch(() => vueDemi.unref(options), handleOptionsChange),
    true
  );
  if (vueDemi.isRef(options)) {
    watchProps();
  }
  return () => {
    stopWatchProps();
    stopWatchElements();
    clearAutoUpdate();
  };
}
function useFloatingOptionsChange(options, onChange, watchOptions) {
  let lastOptions = null;
  const updateLastProps = (options2) => {
    lastOptions = {
      ...options2,
      middleware: options2.middleware ? [...options2.middleware] : []
    };
  };
  const handlePropsChange = (options2) => {
    if (!lastOptions || !equalFloatingProps(lastOptions, options2)) {
      updateLastProps(options2);
      onChange(options2);
    }
  };
  const { clear: stop, mesure } = useManualEffect(
    () => vueDemi.watch(options, handlePropsChange, watchOptions),
    true
  );
  return {
    stop,
    mesure
  };
}
function equalFloatingProps(a, b) {
  return a.strategy === b.strategy && a.placement === b.placement && !!a.disabled === !!b.disabled && equalMiddlewares(a.middleware, b.middleware);
}
function equalMiddleware(a, b) {
  return a.name === b.name && a.options === b.options && a.fn === b.fn;
}
function equalMiddlewares(a, b) {
  if (!a || !b || a.length !== b.length) {
    return false;
  }
  const remainings = [...b];
  let i = -1;
  while (++i < a.length) {
    const index = remainings.findIndex((r) => equalMiddleware(a[i], r));
    if (index === -1) {
      return false;
    }
    remainings.splice(index, 1);
  }
  return true;
}

// src/useFloating.ts
function useFloating(referenceRef, floatingRef, options) {
  const optionsRef = vueDemi.computed(() => {
    const userOptions = vueDemi.unref(options) || {};
    return {
      ...userOptions,
      strategy: userOptions.strategy || "absolute",
      placement: userOptions.placement || "bottom",
      middleware: userOptions.middleware
    };
  });
  const dataRef = vueDemi.ref({
    x: null,
    y: null,
    strategy: optionsRef.value.strategy,
    placement: optionsRef.value.placement,
    middlewareData: {}
  });
  const updatePosotion = () => {
    const reference = vueDemi.unref(referenceRef);
    const floating = vueDemi.unref(floatingRef);
    if (reference && floating) {
      dom.computePosition(reference, floating, optionsRef.value).then((data) => {
        dataRef.value = data;
      });
    }
  };
  const {
    detect: detectElements,
    mesure: watchElements,
    stop: stopWatchElements
  } = useQualifiedRefs(
    [referenceRef, floatingRef],
    (qualifys) => qualifys && updatePosotion()
  );
  let disabled = false;
  const handleOptionsChange = (options2) => {
    const lastDisabled = disabled;
    disabled = !!options2.disabled;
    if (disabled !== lastDisabled) {
      if (!disabled) {
        detectElements();
        watchElements();
      } else {
        stopWatchElements();
      }
    } else if (!disabled) {
      updatePosotion();
    }
  };
  const { stop: stopWatchProps } = useFloatingOptionsChange(
    optionsRef,
    handleOptionsChange,
    {
      immediate: true
    }
  );
  return {
    data: dataRef,
    update: updatePosotion,
    stop: () => {
      stopWatchProps();
      stopWatchElements();
    }
  };
}

exports.createEmptyEffect = createEmptyEffect;
exports.useAutoUpdate = useAutoUpdate;
exports.useFloating = useFloating;
exports.useFloatingOptionsChange = useFloatingOptionsChange;
exports.useManualEffect = useManualEffect;
exports.useQualifiedRefs = useQualifiedRefs;
